// transportclient.cpp : 
//

#include "stdafx.h"
#include <windows.h>
#include "stdlib.h"
#include "string.h"
#include "transportclient.h"

unsigned char key[]="TESTKEY";
int keylen = 7;

int exit_flag = 0;

/* represet a ICE session */
ICESocket *g_ice_socket = NULL;
/* clinet endpoint it connect to the server */
TrAsyncSocketClient *g_client_socket = NULL;

/*
 * After the connection established between two clients
 * send test packets through ICE session each other
 */
unsigned int ProcMsgThreadEntry(LPVOID lpParam)
{	
	char pktbuf[1024];
	int  seq = 1;

	while(!exit_flag)
	{
		sprintf(pktbuf, "p2p test msg ~v~ :) 1234567890asdfghjkl seq=%d", seq++);
		g_ice_socket->SendPacketData(pktbuf, strlen(pktbuf)+1);

		Sleep(30);
	}

	return 0;
}

int main(int argc, char* argv[])
{
	TrSocketAddress svr_addr;
	TrSocketAddress stun_addr;
	TrSocketAddress relay_addr;

	char server_ip[MAX_HOST_NAME_LENGTH];
	char ch;
	bool ret = false;
	
	/*
	 * Input the server IP address
	 */
	memset(server_ip, 0, MAX_HOST_NAME_LENGTH);
	printf("Please input server ip[<%d char]:\n", MAX_HOST_NAME_LENGTH);
	fscanf(stdin, "%s", server_ip);

	/*
	 * This client want to connect to STUN and Relay servers
	 * These servers help clients to traverse NATs
	 */
	strncpy(stun_addr.host, server_ip, MAX_HOST_NAME_LENGTH);
	stun_addr.port = 7000;

	strncpy(relay_addr.host, server_ip, MAX_HOST_NAME_LENGTH);
	relay_addr.port = 5000;

	/*
	 * Firstly initialize the SDK
	 */
	Tr_Init(&stun_addr, &relay_addr);

	/*
	 * Create a new client socket
	 */
	g_client_socket = new TrAsyncSocketClient(TR_SOCK_STREAM);

	/*
	 * The client connect to the server
	 */
	strncpy(svr_addr.host, server_ip, MAX_HOST_NAME_LENGTH);
	svr_addr.port = 9000;
	g_client_socket->Connect(&svr_addr);

	while(1){

		printf("Enter q to exit program:\n");
		ch = fgetc(stdin);
		if (ch == 'q'){
			exit_flag = 1;
			break;
		}
		
	}

	g_client_socket->Close();
	delete g_client_socket;

	Tr_Destroy();

	return 0;
}

/*
 * Create a ICE socket (session)
 * if first parameter is true then this session support reliable packet transmit
 * if keylen is not zero it means the session must encrypt/decrypt packet data
 * using key string
 */
ICESocket::ICESocket():TrICESocket(true, key, keylen){}
ICESocket::~ICESocket(){}

/*
 * ICE socket receive a packet from remote endpoint
 */
void 
ICESocket::OnReceivePacketData(const char *data, int len)
{
	printf("Recv data [%s] len=%d\n", data, len);
	return;
}

void 
ICESocket::OnSessionState(TrSessionState state)
{	
	DWORD	dwThreadId;
	HANDLE  hHandle;

	switch(state)
	{
	case TR_STATE_SESSION_CREATED:
		printf("State:TR_STATE_SESSION_CREATED\n");
		break;
	case TR_STATE_SESSION_DESTROYED:
		printf("State:TR_STATE_SESSION_DESTROYED\n");
		break;
	case TR_STATE_CONNECTING:
		printf("State:TR_STATE_CONNECTING\n");
		break;
	case TR_STATE_CONNECTED:
		printf("State:TR_STATE_CONNECTED\n");
		/*
		 * ICE socket connect to remote endpoint succefull
		 * this means two clients behinded NATs have connected by UDP or STUN UDP or ...
		 */
		hHandle = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE)ProcMsgThreadEntry, (LPVOID)NULL, 0, &dwThreadId);
		CloseHandle(hHandle);
		break;
	case TR_STATE_NETWORK_ERROR:
		printf("State:TR_STATE_NETWORK_ERROR\n");
		break;
	}
}

/*
 * This ICE socket has create a port(UDP/TCP/STUN/RELAY) and is ready 
 * for remote connecting
 * SDK report this msg to the App, App must through control signal channel
 * send it to remote endpoint
 */
void 
ICESocket::OnCandidatesReady(TrCandidate *candidates, int count)
{
	char pktbuf[1024];
	int  i;

	printf("=====Candidates Ready====\n");

	for(i=0; i<count; i++){
		TrCandidate *c = (TrCandidate *)candidates+i;
		
		memset(pktbuf, 0, sizeof(pktbuf));

		sprintf(pktbuf, "%s %s %s %d %f %s %s %s %s %d",
			c->name,
			c->protocol,
			c->address.host,
			c->address.port,
			c->preference,
			c->username,
			c->password,
			c->type,
			c->network_name,
			c->generation);
		printf("candidate[%d]:%s\n", i, pktbuf);

		/*
		 * HERE!! Send candidate info to the server
		 */
		g_client_socket->Write(pktbuf, strlen(pktbuf)+1);
	}

	return;
}

TrAsyncSocketClient::TrAsyncSocketClient(int socket_type): TrAsyncSocket(socket_type){};
TrAsyncSocketClient::~TrAsyncSocketClient(){};

void 
TrAsyncSocketClient::OnSignalEvent(State s){ 

	switch(s){
		case STATE_CLOSED:
			printf("State: STATE_CLOSED\n");
			break;
		case STATE_OPEN:
			printf("State: STATE_OPEN\n");	
			break;
	}
}

/*
 * The client receive control command or remote candidate 
 * info from the server 
 */
void 
TrAsyncSocketClient::OnRead(char * data, int len){
	char name[64];
	char protocol[16];
	char str_port[8];
	TrSocketAddress address;
	char str_preference[16];
	char username[128];
	char password[128];
	char type[16];
	char network_name[64];
	char str_generation[8];
	TrCandidate c;

	printf("Receive data: %s size: %d\n", data, len);

	/*
	 * In this simple demo, this string must be the control command
	 * sended by the server, so the client must quickly create a ICE
	 * socket for Peer to Peer connecting
	 */
	if(len==(strlen("START")+1) && !strcmp(data, "START")){

		printf("Creating ICESocket...\n");
		g_ice_socket = new ICESocket();
		Tr_CreateICESocket(g_ice_socket);
		return;
	}

	/*
	 * If the msg is not start command it must be a candidate info 
	 * forward by the server from the remote endpoint
	 */
	sscanf(data, "%s%s%s%s%s%s%s%s%s%s",
			name,
			protocol,
			address.host,
			str_port,
			str_preference,
			username,
			password,
			type,
			network_name,
			str_generation);

	c.name = name;
	c.protocol = protocol;
	strcpy(c.address.host, address.host);
	c.address.port = atoi(str_port);
	c.preference = atof(str_preference);
	c.username = username;
	c.password = password;
	c.type = type;
	c.network_name = network_name;
	c.generation = atoi(str_generation);

	/*
	 * Tell the SDK and the ICE socket know where 
	 * it will connect to
	 */
	g_ice_socket->AddRemoteCandidates(&c, 1);

};



